@@ -107,6 +107,26 @@ def fetch_crypto_daily_data(ssj=None, local_root_directory=None, daily_return_pk
         dataframe = pd.read_pickle(pickle_saving_path)
     return dataframe
 
+
+def request_minute_data_paquet(url, me_ts, ssj):
+    r = requests.get(url.format(ssj, me_ts))
+    dataframe = None
+    try:
+        dataframe = pd.DataFrame(json.loads(r.text)['Data']['Data'])
+    except Exception as e:
+        print('no data')
+    return dataframe
+
+def request_hour_data_paquet(url, me_ts, ssj):
+    r = requests.get(url.format(ssj, me_ts))
+    dataframe = None
+    try:
+        dataframe = pd.DataFrame(json.loads(r.text)['Data'])
+    except Exception as e:
+        print('no data')
+    return dataframe
+
+
 def fetch_crypto_minutely_data(ssj=None, local_root_directory=None, minutely_return_pkl_filename_suffix=None, running_date= None, daily_crypto_starting_day='2012-01-01', daily_crypto_ending_day=None, refetch_all=True, ssj_against='USDT', exchange = 'binance'):
     dates_stub = daily_crypto_starting_day.strftime('%d_%b_%Y') + '_' + daily_crypto_ending_day.strftime('%d_%b_%Y')
     pickle_saving_path = local_root_directory+ssj+'_'+dates_stub+minutely_return_pkl_filename_suffix
@@ -130,74 +150,16 @@ def fetch_crypto_minutely_data(ssj=None, local_root_directory=None, minutely_ret
         ts12 = ts11 - 2001 * 60
         print('Loading data')
         binance_usdt_url = 'https://min-api.cryptocompare.com/data/v2/histominute?fsym={}&tsym='+ssj_against+'&e='+exchange+'&toTs={}&limit=2000'
-        r = requests.get(
-            binance_usdt_url.format(ssj, ts12))
-        dataframe = pd.DataFrame(json.loads(r.text)['Data'])
-        print('waiting')
-        time.sleep(1)
-        r = requests.get(
-            binance_usdt_url.format(ssj, ts11))
-        df = pd.DataFrame(json.loads(r.text)['Data'])
-        dataframe = dataframe.append(df, ignore_index=True)
-        print('waiting')
-        time.sleep(1)
-        r = requests.get(
-            binance_usdt_url.format(ssj, ts10))
-        df = pd.DataFrame(json.loads(r.text)['Data'])
-        dataframe = dataframe.append(df, ignore_index=True)
-        print('waiting')
-        time.sleep(1)
-        r = requests.get(
-            binance_usdt_url.format(ssj, ts9))
-        df = pd.DataFrame(json.loads(r.text)['Data'])
-        dataframe = dataframe.append(df, ignore_index=True)
-        print('waiting')
-        time.sleep(1)
-        r = requests.get(
-            binance_usdt_url.format(ssj, ts8))
-        df = pd.DataFrame(json.loads(r.text)['Data'])
-        dataframe = dataframe.append(df, ignore_index=True)
-        print('waiting')
-        time.sleep(1)
-        r = requests.get(
-            binance_usdt_url.format(ssj, ts7))
-        df = pd.DataFrame(json.loads(r.text)['Data'])
-        dataframe = dataframe.append(df, ignore_index=True)
-        print('waiting')
-        time.sleep(1)
-        r = requests.get(
-            binance_usdt_url.format(ssj, ts6))
-        df = pd.DataFrame(json.loads(r.text)['Data'])
-        dataframe = dataframe.append(df, ignore_index=True)
-        print('waiting')
-        time.sleep(1)
-        r = requests.get(
-            binance_usdt_url.format(ssj, ts5))
-        df = pd.DataFrame(json.loads(r.text)['Data'])
-        dataframe = dataframe.append(df, ignore_index=True)
-        print('waiting')
-        time.sleep(1)
-        r = requests.get(
-            binance_usdt_url.format(ssj, ts4))
-        df = pd.DataFrame(json.loads(r.text)['Data'])
-        dataframe = dataframe.append(df, ignore_index=True)
-        time.sleep(1)
-        r = requests.get(
-            binance_usdt_url.format(ssj, ts3))
-        df = pd.DataFrame(json.loads(r.text)['Data'])
-        dataframe = dataframe.append(df, ignore_index=True)
-        r = requests.get(
-            binance_usdt_url.format(ssj, ts2))
-        df = pd.DataFrame(json.loads(r.text)['Data'])
-        dataframe = dataframe.append(df, ignore_index=True)
-        r = requests.get(
-            binance_usdt_url.format(ssj, ts1))
-        df = pd.DataFrame(json.loads(r.text)['Data'])
-        dataframe = dataframe.append(df, ignore_index=True)
-        r = requests.get(
-            binance_usdt_url.format(ssj, ts))
-        df = pd.DataFrame(json.loads(r.text)['Data'])
-        dataframe = dataframe.append(df, ignore_index=True)
+
+        dataframe = None
+        for me_timestamp in [ts12,ts11,ts10,ts9,ts8,ts7,ts6,ts5,ts4,ts3,ts2,ts1,ts]:
+            print('waiting')
+            df = request_minute_data_paquet(binance_usdt_url, me_timestamp, ssj)
+            if df is not None:
+                if dataframe is not None:
+                    dataframe = dataframe.append(df, ignore_index=True)
+                else:
+                    dataframe = df.copy()
 
         dataframe['time'] = pd.to_datetime(dataframe['time'], unit='s')
         dataframe = dataframe.sort_values(by=['time'])
@@ -236,78 +198,15 @@ def fetch_crypto_hourly_data(ssj=None, local_root_directory=None, hourly_return_
         ts13 = ts12 - 2001 * 3600
         print('Loading data')
         hours_url_request = 'https://min-api.cryptocompare.com/data/histohour?fsym={}&tsym=' + ssj_against + '&e=' + exchange + '&toTs={}&limit=2000'
-        r = requests.get(
-            hours_url_request.format(ssj, ts13))
-        dataframe = pd.DataFrame(json.loads(r.text)['Data'])
-        print('waiting')
-        time.sleep(1)
-        r = requests.get(
-            hours_url_request.format(ssj, ts12))
-        df = pd.DataFrame(json.loads(r.text)['Data'])
-        dataframe = dataframe.append(df, ignore_index=True)
-        print('waiting')
-        time.sleep(1)
-        r = requests.get(
-            hours_url_request.format(ssj, ts11))
-        df = pd.DataFrame(json.loads(r.text)['Data'])
-        dataframe = dataframe.append(df, ignore_index=True)
-        print('waiting')
-        time.sleep(1)
-        r = requests.get(
-            hours_url_request.format(ssj, ts10))
-        df = pd.DataFrame(json.loads(r.text)['Data'])
-        dataframe = dataframe.append(df, ignore_index=True)
-        print('waiting')
-        time.sleep(1)
-        r = requests.get(
-            hours_url_request.format(ssj, ts9))
-        df = pd.DataFrame(json.loads(r.text)['Data'])
-        dataframe = dataframe.append(df, ignore_index=True)
-        print('waiting')
-        time.sleep(1)
-        r = requests.get(
-            hours_url_request.format(ssj, ts8))
-        df = pd.DataFrame(json.loads(r.text)['Data'])
-        dataframe = dataframe.append(df, ignore_index=True)
-        print('waiting')
-        time.sleep(1)
-        r = requests.get(
-            hours_url_request.format(ssj, ts7))
-        df = pd.DataFrame(json.loads(r.text)['Data'])
-        dataframe = dataframe.append(df, ignore_index=True)
-        print('waiting')
-        time.sleep(1)
-        r = requests.get(
-            hours_url_request.format(ssj, ts6))
-        df = pd.DataFrame(json.loads(r.text)['Data'])
-        dataframe = dataframe.append(df, ignore_index=True)
-        print('waiting')
-        time.sleep(1)
-        r = requests.get(
-            hours_url_request.format(ssj, ts5))
-        df = pd.DataFrame(json.loads(r.text)['Data'])
-        dataframe = dataframe.append(df, ignore_index=True)
-        time.sleep(1)
-        r = requests.get(
-            hours_url_request.format(ssj, ts4))
-        df = pd.DataFrame(json.loads(r.text)['Data'])
-        dataframe = dataframe.append(df, ignore_index=True)
-        r = requests.get(
-            hours_url_request.format(ssj, ts3))
-        df = pd.DataFrame(json.loads(r.text)['Data'])
-        dataframe = dataframe.append(df, ignore_index=True)
-        r = requests.get(
-            hours_url_request.format(ssj, ts2))
-        df = pd.DataFrame(json.loads(r.text)['Data'])
-        dataframe = dataframe.append(df, ignore_index=True)
-        r = requests.get(
-            hours_url_request.format(ssj, ts1))
-        df = pd.DataFrame(json.loads(r.text)['Data'])
-        dataframe = dataframe.append(df, ignore_index=True)
-        r = requests.get(
-            hours_url_request.format(ssj, ts))
-        df = pd.DataFrame(json.loads(r.text)['Data'])
-        dataframe = dataframe.append(df, ignore_index=True)
+        dataframe = None
+        for me_timestamp in [ts13, ts12, ts11, ts10, ts9, ts8, ts7, ts6, ts5, ts4, ts3, ts2, ts1, ts]:
+            print('waiting')
+            df = request_hour_data_paquet(hours_url_request, me_timestamp, ssj)
+            if df is not None:
+                if dataframe is not None:
+                    dataframe = dataframe.append(df, ignore_index=True)
+                else:
+                    dataframe = df.copy()
 
         dataframe['time'] = pd.to_datetime(dataframe['time'], unit='s')
         dataframe = dataframe.sort_values(by=['time'])
